import io
import csv
import json
import time
import avro.schema
from avro.io import DatumWriter
from datetime import datetime
from confluent_kafka import Producer

def getTime(str):
    if ( len(str) == 10 ):
         str = str + " 00:00"
    return int(datetime.strptime(str,'%d/%m/%Y %H:%M').timestamp())

def buildJSON(row,id):
    return {
        "id": id,
        "startime": getTime(row[0]),
        "endtime": getTime(row[1]),
        "base": row[2],
        "userid": row[3]
    }

def main():
    count = 0
    topic = "calls"
    schemaPath = './resources/call.avsc'
    schema = avro.schema.parse(open(schemaPath, 'r').read())
    producer = Producer( {'streams.producer.default.stream': '/telecom/mystream'} )

    with open('./data/calls.csv', 'r') as csvfile:
        reader = csv.reader(csvfile, delimiter=',')
        header = next(reader)
        for row in reader:
            try:
                data = buildJSON(row,str(count))

                writer = DatumWriter(schema)
                bytesWriter = io.BytesIO()
                encoder = avro.io.BinaryEncoder(bytesWriter)
                writer.write( data, encoder)

                producer.produce( topic, key=data['id'], value=bytesWriter.getvalue()  )
                print(count, end='\r')
                count = count + 1
            except Exception as err:
                print(err)

        producer.flush()
        print("All calls sent to topic")
        print(count)

if __name__ == "__main__":
    main()



from kafka import KafkaProducer
from kafka import KafkaConsumer


def kafka_publish(host: str, topic: str, messages: list):
    producer = KafkaProducer(
        bootstrap_servers=[f"{host}:9092"],
        security_protocol="SASL_PLAINTEXT",
        sasl_mechanism="PLAIN",
        sasl_plain_username="mapr",
        sasl_plain_password="mapr",
        
    )

    logging.info(f"Sending {len(messages)} messages to {topic}")

    for msg in messages:
        try:
            future = producer.send(topic, json.dumps(msg).encode())
            result = future.get(timeout=60)
            print("DEBUG FUTURE", result)
            metrics = producer.metrics()
            print("DEBUG METRICS", metrics)

        except Exception as error:
            logging.warning(error)

        finally:
            app.storage.general["demo"]["counting"] += 1

    producer.flush()


def kafka_consume(host: str, topic: str):

    consumer = KafkaConsumer(bootstrap_servers=[f"{host}:9092"],
                            auto_offset_reset='earliest',
                            security_protocol='SASL_PLAINTEXT',
                            sasl_mechanism='PLAIN',
                            sasl_plain_username='mapr',
                            sasl_plain_password='mapr')

    consumer.subscribe([topic])
    numMsgConsumed = 0
    for _ in range(10):
        records = consumer.poll(timeout_ms=500)
        for topic_data, consumer_records in records.items():
            print(f"ERDINC: {topic_data}: {consumer_records}")
            for consumer_record in consumer_records:
                print("Received message: " + str(consumer_record.value.decode('utf-8')))
                numMsgConsumed += 1
    print("Messages consumed: " + str(numMsgConsumed))



from datetime import datetime
import logging
from nicegui import app, ui, run, native
import inspect

import requests

from ezshow import apprunner
from ezshow import shrunner
from ezshow import restrunner
import ezshow
from ezlab.utils import demos, demopath

logger = logging.getLogger("ezshow")
# Disable annoying messages in log view
# Thanks to: https://sam.hooke.me/note/2023/10/nicegui-binding-propagation-warning/
import nicegui.binding


# Increase threshold for binding propagation warning from 0.01 to 0.02 seconds
nicegui.binding.MAX_PROPAGATION_TIME = 0.03

if not demos:
    print("Failed to load demos!")
    exit(1)


@ui.page("/")
def menu():
    # initialize app storage for demos
    if "ui" not in app.storage.general.keys():
        app.storage.general["ui"] = {}
    if "demo" not in app.storage.general.keys():
        app.storage.general["demo"] = {}

    appstore = app.storage.general["demo"]

    app.storage.user["busy"] = False

    with (
        ui.expansion(
            "Demos",
            icon="apps",
            caption="guided demos",
        )
        .classes("w-full")
        .classes("text-bold") as ezshow_menu
    ):
        # Connect to a cluster
        ui.label("Use cluster")
        with ui.row().classes("w-full"):
            # ui.input("Cluster Name", placeholder="edf-core").bind_value(
            #     appstore, "clustername"
            # )
            ui.input("Host", placeholder="node1.cluster.lab").bind_value(
                appstore, "host"
            )
            ui.input("Username", placeholder="mapr").bind_value(appstore, "username")
            ui.input(
                "Password",
                placeholder="mapr",
                password=True,
                password_toggle_button=True,
            ).bind_value(appstore, "password")
            # ui.button(
            #     "Login",
            #     on_click=login,
            # ).bind_enabled_from(
            #     app.storage.user, "busy", backward=lambda x: not x
            # ).bind_visibility_from(app.storage.user, "cookiejar", lambda x: x is None)

        demo_tabs = {}
        # Demos (Tabs)
        with ui.tabs().classes("w-full") as tabs:
            for demo in demos:
                demo_tabs[demo["name"]] = ui.tab(demo["name"])

        # Demo content
        with (
            ui.tab_panels(tabs)
            .classes("w-full")
            .bind_value(app.storage.general["demo"], "name")
        ):
            for demo in demos:

                # Demo panel
                with ui.tab_panel(demo_tabs[demo["name"]]):
                    ui.markdown(demo["description"]).classes("font-normal")
                    ui.separator()
                    ui.image(
                        f"{demopath}/{demo['name'].replace(' ', '-')}/{demo['diagram']}"
                    ).classes("object-scale-down g-10")

                    ui.link(
                        "Source",
                        target=demo.get("link", "https://github.com/mapr-demos/"),
                        new_tab=True,
                    ).bind_visibility_from(
                        demo, "link", backward=lambda x: x is not None
                    )
                    ui.separator()
                    ui.label("Follow the step by step demo flow")
                    with ui.row():
                        ui.label("Using volume:")
                        ui.label(demo.get("volume", "/"))

                    # Steps
                    with (
                        ui.stepper()
                        .props("vertical header-nav")
                        .classes("w-full") as stepper
                    ):
                        for step in demo["steps"]:

                            # Step #
                            with ui.step(step["id"], title=step["name"].title()).props(
                                f"active-icon=play_arrow caption=\"using {step['runner']}\""
                            ):
                                ui.markdown(step["description"]).classes("font-normal")
                                if step["runner"] == "rest":
                                    ui.label(
                                        f"https://{appstore['host']}:8443{step['runner_parameters']}"
                                    )

                                elif step['runner'] == 'app':
                                    # Display source code if 'codes' provided
                                    if 'codes' in step.keys():
                                        for c in step['codes']:
                                            t = c.split('.')
                                            module_name=getattr(ezshow, t[0])
                                            function_name=t[1]

                                            ui.code(
                                                inspect.getsource(
                                                    getattr(
                                                        module_name,
                                                        function_name
                                                    )
                                                )
                                            ).classes("w-full")
                                    else:
                                        ui.label(step["runner_parameters"])

                                    # Display function params if 'count' is provided
                                    if 'count' in step.keys():
                                        ui.label().bind_text_from(
                                            app.storage.user,
                                            f"count__{demo['name'].replace(' ', '_')}__{step['id']}",
                                            # format message to show function(param, count) format
                                            backward=lambda x, y=step.get(
                                                "app_parameters", None
                                            ), p=step["runner_parameters"]: f"Will be running: {p}( {y}, {x} )"
                                        )
                                    else:
                                        ui.label(f"Will be running: {step['runner_parameters']} ( {step.get('app_parameters', None)} )" )

                                # Insert count slider if step wants one
                                if step.get("count", None):
                                    with ui.row().classes("w-full"):
                                        slider = ui.slider(
                                            min=0,
                                            max=3*step['count'],
                                            step=1,
                                            value=step["count"],
                                            # saving selection in user storage, with demo name (spaces substituted with _), step id and parameter
                                        ).bind_value_to(
                                            app.storage.user,
                                            f"count__{demo['name'].replace(' ', '_')}__{step['id']}",
                                        ).classes("w-5/6 self-center")
                                        ui.label().bind_text_from(
                                            slider, "value"
                                        ).classes("self-center")

                                # Get input if step wants one
                                if step.get("input", None):
                                    ui.input(
                                        step["input"],
                                        placeholder="We need this input to proceed",
                                    ).bind_value_to(
                                        app.storage.user,
                                        # saving selection in user storage, with demo name (spaces substituted with _), step id and parameter
                                        f"input__{demo['name'].replace(' ', '_')}__{step['id']}",
                                    ).classes(
                                        "w-full"
                                    )

                                with ui.stepper_navigation():
                                    ui.button(
                                        "Run",
                                        icon="play_arrow",
                                        on_click=lambda demo=demo, step=step, pager=stepper: run_step(
                                            demo, step, pager
                                        ),
                                    ).bind_enabled_from(
                                        app.storage.user,
                                        "busy",
                                        backward=lambda x: not x,
                                    ).bind_visibility_from(step, "runner", backward=lambda x: x != "noop")
                                    ui.button(
                                        "Next",
                                        icon="fast_forward",
                                        on_click=stepper.next,
                                    ).props("color=secondary flat")
                                    ui.button(
                                        "Back",
                                        icon="fast_rewind",
                                        on_click=stepper.previous,
                                    ).props("flat").bind_visibility_from(
                                        step, "id", backward=lambda x: x != 1
                                    )  # don't show for the first step

                    if "monitors" in demo.keys():
                        for mon in demo['monitors']:
                            mon_chart = ui.echart(
                                {
                                    "tooltip": {
                                        "trigger": "axis",
                                        # "axisPointer": {"type": "cross"},
                                    },
                                    # "toolbox": {
                                    #     "show": "true",
                                    #     "right": 10,
                                    #     "feature": {
                                    #         "saveAsImage": {},
                                    #     },
                                    # },
                                    "title": {"left": 10, "text": ""},
                                    "legend": {"right": "center"},
                                    "xAxis": {
                                        "type": "category",
                                        "boundaryGap": False,
                                        "axisLine": {"onZero": "true"},
                                        "splitLine": {"show": "false"},
                                        "data": [],
                                    },
                                    "yAxis": [
                                        {
                                            "type": "value",
                                            "name": "Count",
                                            "boundaryGap": [0, "100%"],
                                        },
                                        {
                                            "type": "value",
                                            "name": "Milliseconds",
                                            "axisLabel": {
                                                "formatter": "{value} s",
                                            },
                                            "boundaryGap": [0, "100%"],
                                        },
                                    ],
                                    "series": [  # manually set max series to display (TODO: find a pythonic way)
                                        {
                                            "type": "line",
                                            "smooth": True,
                                            "data": [],
                                        },
                                        {
                                            "type": "line",
                                            "symbol": "triangle",
                                            "smooth": True,
                                            "data": [],
                                        },
                                        {
                                            "type": "line",
                                            "symbol": "roundRect",
                                            "smooth": True,
                                            "data": [],
                                        },
                                        {
                                            "type": "line",
                                            "symbol": "pin",
                                            "smooth": True,
                                            "data": [],
                                        },
                                    ],
                                },
                            )
                            # MONITOR FUNCTION SHOULD RETURN
                            # "name": dict_item["type"],
                            # "time": dict_item["time"],
                            # "values": [
                            #     {"Tx": dict_item["tx"]},
                            #     {"Rx": dict_item["rx"]},
                            # ],
                            def add_metric(monitor_name, chart_name):
                                t = monitor_name.split('.')
                                module_name=getattr(ezshow, t[0])
                                function_name=t[1]
                                function_param=t[2]
                                func = getattr(module_name, function_name)

                                # collect the metrics
                                metric = func(function_param)

                                chart_name.options['xAxis']['data'].append(metric['time'])
                                chart_name.options['title']['text'] = metric['name'].title()

                                for idx, serie in enumerate(metric["values"]):
                                    chart_series = chart_name.options['series'][idx]
                                    for key in serie.keys():
                                        if not chart_series.get('name', None):
                                            chart_series['name'] = key
                                        # if name ends with (s), place it onto second yAxis
                                        if "(s)" in key:
                                            chart_series["yAxisIndex"] = 1
                                        chart_series['data'].append(int(serie[key]))
                                chart_name.update()

                            # Using lambda below so we capture the function name for individual steps
                            timer = ui.timer(interval=5.0, callback=lambda monitor=mon, chart=mon_chart: add_metric(monitor, chart), active=False)
                            ui.switch("->".join(mon.split('.')[1:]).title().replace('_', ' ')).bind_value(timer, "active")

    ezshow_menu.bind_value(app.storage.general["ui"], "ezshow")


# async def login():
#     app.storage.user["busy"] = True
#     response = await run.io_bound(rest.get_session)
#     if isinstance(response, Exception):
#         ui.notify(response, type='negative')
#     else:
#         if response.ok:
#             app.storage.user["cookiejar"] = response.cookies.get_dict()
#         else:
#             ui.notify(message=response.text, html=True, type="warning")
#     app.storage.user["busy"] = False


async def run_step(demo, step, pager: ui.stepper):
    app.storage.user["busy"] = True

    if step["runner"] == "rest":
        response = await run.io_bound(restrunner.post, step["runner_parameters"])
        if isinstance(response, Exception):
            ui.notify(response, type="negative")
        elif response.ok:
            resjson = response.json()
            # print("DEBUG RESTRUNNER", resjson)
            # ts = resjson["timestamp"] / 1000 # ignore nanoseconds precision from timestamp
            # st = resjson.get("status", None)
            # ms = resjson.get("messages", ["ERROR"]).pop()
            # er = resjson.get("errors", None)
            # ui.notify(
            #     f"{datetime.fromtimestamp(ts).strftime('%X')} : {ms if 'OK' in st else ' | '.join([e['desc'] for e in er])}",
            #     type="positive" if "OK" in st else "warning",
            # )
            # I took the lazy approach here since different rest calls have different return formats (except the 'status')
            ui.notify(
                resjson, type="positive" if resjson["status"] == "OK" else "warning"
            )
            pager.next()

        else:  # http error returned
            ui.notify(message=response.text, html=True, type="warning")

    elif step["runner"] == "restfile":
        for response in await run.io_bound(
            restrunner.postfile, demos, step["runner_parameters"]
        ):
            if isinstance(response, Exception):
                ui.notify(response, type="negative")
            elif response.ok:
                try:
                    # logging.debug("DEBUG: RESPONSE FROM RESTRUNNER: %s", response)
                    if response.status_code == 201:
                        ui.notify("Folder created")
                    else:
                        resjson = response.json()
                        ui.notify(resjson)
                except Exception as error:
                    print(error)

                pager.next()
            else:  # http error returned
                ui.notify(message=response.text, html=True, type="warning")

    elif step["runner"] == "shell":
        ui.notify(shrunner.run_command(step["runner_parameters"]))
        pager.next()

    elif step["runner"] == "app":
        func = getattr(apprunner, step["runner_parameters"])

        if "count" in step.keys():
            # Reset the counter
            app.storage.general["demo"]["counting"] = 0
            # keep user selected count in sync
            count = app.storage.user[f"count__{demo['name'].replace(' ', '_')}__{step['id']}"]

            # add progressbar if 'count'ing
            ui.linear_progress().bind_value_from(
                app.storage.general["demo"],
                "counting",
                backward=lambda x: f"{100 * int(x/count)} %",
            )
            
            await run.io_bound(func, step["app_parameters"], count)

        else:
            await run.io_bound(func, step["app_parameters"])

        # pager.next()

    else:
        ui.notify(f"Would run {step['runner_parameters']} using {step['runner']}")
        pager.next()

    app.storage.user["busy"] = False


# Entry point for the module
def enter():
    print("EZSHOW RUNNING AS MODULE")
    ui.run(
        title="Ezlab",
        dark=None,
        native=True,
        window_size=(1024, 1024),
        # frameless=True,
        storage_secret="ezmeralr0cks",
        reload=False,
        port=native.find_open_port(),
    )


# For development and debugs
if __name__ in {"__main__", "__mp_main__"}:
    print("EZSHOW RUNNING IN DEV MODE")
    # INSECURE REQUESTS ARE OK in DEV
    requests.packages.urllib3.disable_warnings()

    ui.run(
        title="Ezshow",
        dark=None,
        native=True,
        window_size=(800, 800),
        # frameless=True,
        storage_secret="ezmeralr0cks",
        reload=True,
        port=native.find_open_port(),
    )
    logging.basicConfig(level=logging.DEBUG)



"""
Data Fabric operations user interface
"""

import re
import nicegui
import random
import string
import time

from ezlab.ezmeral.ezdf import client_install, cluster_install, xcluster

from ezlab.parameters import DF
from ezinfra.queue import process_queue
from ezinfra.vms import prepare
from ezlab.utils import ezapp

appstore = nicegui.app.storage.general

@nicegui.ui.refreshable
def add_remove_host(hosts: list):
    if hosts is None:
        hosts = []
    for host in hosts:
        with nicegui.ui.row().classes("w-full items-center no-wrap"):
            nicegui.ui.input(
                "Hostname",
                placeholder="Host shortnames, ie, ezvm1,ezvm2...",
            ).bind_value(host, "name")
            nicegui.ui.input(
                "IP Address",
                placeholder="10.1.1.x",
            ).bind_value(host, "ip")


def get_doc(content: dict):

    random_eof = "".join(random.choices(string.ascii_uppercase + string.digits, k=10))

    def indent(strings: str, spaces: int):
        return strings.strip().replace("\n", f"\n{' '*spaces}")

    settings = ""

    for key, value in content["settings"].items():
        settings += f"""
    #   {key}: {value if len(str(value)) < 80 else value[:77] + '...'}"""

    files = ""
    for filepath, filecontent in content["files"].items():
        files += f"""
    sudo cat <<{random_eof}>{filepath}
    {indent(filecontent,4)}
    {random_eof}
    """

    commands = ""
    for command in content["commands"]:
        commands += f"""
    {indent(command,4)}
    """

    doc = f"""
    #!/usr/bin/env bash

    # Ezlab Run {time.strftime('%Y%m%d%H%M%z')}
    
    # {content["task"]}

    # Settings
        {settings}

    # Write Files
        {files}
    
    # Run Commands
        {commands}

    # End of {content['task']}

"""

    with (
        nicegui.ui.dialog().props("full-width") as dialog,
        nicegui.ui.card().classes("w-full"),
    ):
        nicegui.ui.code(doc, language="bash").classes("w-full text-wrap")
        with nicegui.ui.row():
            nicegui.ui.button(
                "Save (useless on app mode)",
                icon="save",
                on_click=lambda: nicegui.ui.download(doc.encode(), "ezlab-runbook.sh"),
            )
            nicegui.ui.button(
                "Close",
                icon="cancel",
                on_click=dialog.close,
            )
    dialog.open()


async def prepare_action(pager: nicegui.ui.stepper):
    hosts = appstore[DF]["hosts"]
    userstore = nicegui.app.storage.user

    # deep check list[dict] for missing values (if any host's name or IP is missing)
    if not (
        len(hosts) > 0
        and all(
            [values for values in [all(host) for host in [h.values() for h in hosts]]]
        )
    ):
        nicegui.ui.notify("Missing values", type="warning")
        return False

    ezapp.queue.put("Starting configuration for DF")
    for host in hosts:
        result = prepare(
            hostname=host["name"],
            hostip=host["ip"],
            config=dict(appstore["config"]),
            addhosts=[h for h in hosts if h["name"] != host["name"]],
            add_to_noproxy=(
                re.split(
                    ":|/", appstore[DF].get("maprlocalrepo").split("://")[1]
                )[0]
                if appstore[DF].get("maprrepoislocal", False)
                else None
            ),
            prepare_for=DF,
            dryrun=appstore["config"]["dryrun"],
        )
        if appstore["config"]["dryrun"]:
            get_doc(await result)
        elif result:
            pass
        else:
            nicegui.ui.notify(f"[{host}] preparation failed!", type="warning")

    userstore["busy"] = True
    result = await nicegui.run.io_bound(process_queue, len(hosts))
    userstore["busy"] = False

    if result:
        nicegui.ui.notify(f"Finished with {len(hosts)} host(s)", type="positive")
        pager.next()
        return True
    else:
        return False


def prepare_menu():
    if not "hosts" in appstore[DF].keys():
        appstore[DF]["hosts"] = [{"name": "", "ip": ""}]
    with nicegui.ui.row().classes("w-full items-center items-stretch") as prep_view:
        nicegui.ui.label("Hosts")
        nicegui.ui.space()
        with nicegui.ui.row():

            def add_host():
                try:
                    appstore[DF]["hosts"].append({"name": "", "ip": ""})
                except (KeyError, AttributeError):
                    appstore[DF]["hosts"] = [{"name": "", "ip": ""}]
                finally:
                    add_remove_host.refresh(appstore[DF]["hosts"])

            def remove_host():
                if len(appstore[DF]["hosts"]) > 0:
                    appstore[DF]["hosts"].pop()
                    add_remove_host.refresh()

            nicegui.ui.button(icon="add_box", color=None, on_click=add_host).props(
                "flat"
            )
            nicegui.ui.button(icon="delete", color=None, on_click=remove_host).props(
                "flat"
            )

    # Hosts
    add_remove_host(appstore[DF]["hosts"])


async def install_action(hosts_string: str, pager: nicegui.ui.stepper):
    userstore = nicegui.app.storage.user

    ezapp.queue.put("Starting DF cluster installation")

    result = cluster_install(
        hosts=hosts_string.split(","),
        settings=dict(appstore["config"]),
        config=dict(appstore[DF]),
        dryrun=appstore["config"]["dryrun"],
    )

    if appstore["config"]["dryrun"]:
        get_doc(await result)

    elif not result:
        nicegui.ui.notify(f"Cluster creation failed!", type="negative")

    userstore["busy"] = True
    result = await nicegui.run.io_bound(process_queue, 1)
    userstore["busy"] = False
    if result:
        nicegui.ui.notify(f"Finished cluster creation", type="positive")
        pager.next()
        return True
    else:
        return False


async def xcluster_action(pager: nicegui.ui.stepper):
    userstore = nicegui.app.storage.user
    
    ezapp.queue.put("Starting DF cross-cluster setup")

    if not xcluster(
        config=dict(appstore["config"]),
        settings=dict(appstore[DF]),
    ):
        nicegui.ui.notify(f"Cluster creation failed!", type="negative")

    userstore["busy"] = True
    res = await nicegui.run.io_bound(process_queue, 1)
    userstore["busy"] = False
    if res:
        nicegui.ui.notify(f"Finished cross-cluster setup", type="positive")
        pager.next()
        return True
    else:
        return False


async def client_action(pager: nicegui.ui.stepper):
    userstore = nicegui.app.storage.user
    
    ezapp.queue.put("Starting DF client configuration")

    result = client_install(
        username=appstore["config"]["username"],
        keyfile=appstore["config"]["privatekeyfile"],
        config=dict(appstore[DF]),
    )

    if not result:
        nicegui.ui.notify("Client configuration failed", type="negative")

    userstore["busy"] = True
    result = await nicegui.run.io_bound(process_queue, 1)
    userstore["busy"] = False
    if result:
        nicegui.ui.notify(f"Finished client configuration", type="positive")
        pager.next()
        return True
    else:
        return False



import io
import os
import json
import requests
import avro.schema
import avro.io
from confluent_kafka import Consumer, KafkaError

def main():
    running = True
    topic = 'telecom:calls'
    url = 'http://'+ os.environ['MAPR_CLUSTER'] + ':4242/api/put'
    schemaPath = './data/call.avsc'
    consumer = Consumer({'group.id': 'mygroup', 'default.topic.config': {'auto.offset.reset': 'earliest'}} )

    consumer.subscribe([topic])
    schema = avro.schema.parse(open(schemaPath, 'r').read())

    while running:
        msg = consumer.poll(timeout=1.0)
        if msg is None: continue
  
        if not msg.error():  
            bytes_reader = io.BytesIO(msg.value())
            decoder = avro.io.BinaryDecoder(bytes_reader)
            reader = avro.io.DatumReader(schema)
            data = reader.read(decoder)
            
            telemetry = {}
            telemetry['metric'] = "Entries"
            telemetry['timestamp'] = data['startime']
            telemetry['value'] = 1

            tags = {}
            tags['Base'] = data['base'] 
            telemetry['tags'] = tags

            print('Received message: %s' % json.dumps(telemetry))
            requests.post(url, json = telemetry)
        elif msg.error().code() != KafkaError._PARTITION_EOF:
            print(msg.error())
            running = False
    
    consumer.close()


if __name__ == "__main__":
    main()



import csv
import json
import time
from datetime import datetime

def getTime(str):
    if ( len(str) == 10 ):
         str = str + " 00:00"
    return int(datetime.strptime(str,'%d/%m/%Y %H:%M').timestamp())

def buildJSON(row):
    return {
        "startime": getTime(row[0]),
        "endtime": getTime(row[1]),
	"base": row[2],
        "userid": row[3]
    }


def main():
    count = 0
    with open('../data/call.csv', 'r') as csvfile:
        reader = csv.reader(csvfile, delimiter=',')
        header = next(reader)
        for row in reader:
            try:
                json = buildJSON(row)
                count = count + 1
            except Exception as err:
                print(row)
        
        print(count)

if __name__ == "__main__":
    main()



import json
import logging
import pathlib
import requests

from nicegui import app

logger = logging.getLogger("restrunner")

if "demo" not in app.storage.general.keys():
    app.storage.general["demo"] = {}

demostore = app.storage.general["demo"]

AUTH_CREDENTIALS = (demostore["username"], demostore["password"])


def get(path: str):
    REST_URL = f"https://{demostore['host']}:8443{path}"
    try:
        response = requests.get(url=REST_URL, auth=AUTH_CREDENTIALS, verify=False)
        # logger.debug("GET RESPONSE: %s", response.text)
        response.raise_for_status()
        return response

    except Exception as error:
        logger.warning("GET ERROR %s", error)
        return None


def post(path: str):
    REST_URL = f"https://{demostore['host']}:8443{path}"
    # logger.debug("POST URL: %s", REST_URL)
    try:
        response = requests.post(url=REST_URL, auth=AUTH_CREDENTIALS, verify=False)
        # logger.debug("POSTRESPONSE: %s", response)
        response.raise_for_status()
        return response

    except Exception as error:
        logger.warning("POST ERROR %s", error)
        return None


def postfile(demos: list, path: str):

    demo = [demo for demo in demos if demo["name"] == demostore["name"]].pop()

    basedir = (
        f"{pathlib.Path().resolve()}/ezshow/demos/{demo['name'].replace(' ', '-')}"
    )
    fileorfolder = f"{basedir}/{path}"
    volume = demo["volume"]

    if fileorfolder[-1] == "/":
        # Create the folder first
        yield post(f"/files{volume}/{path}")
        source_files = [p for p in pathlib.Path(fileorfolder).iterdir() if p.is_file()]
        target_dir = path
    else:
        source_files = [fileorfolder]
        target_dir = ""

    for file in source_files:
        filename = pathlib.Path(file).name
        destination = volume + "/" + target_dir + filename
        # files = {filename: open(file, "rb")}
        REST_URL = f"https://{demostore['host']}:8443/files{destination}"
        try:
            with open(file, "rb") as f:
                response = requests.put(
                    url=REST_URL,
                    auth=AUTH_CREDENTIALS,
                    verify=False,
                    data=f,
                    timeout=300,
                )
                response.raise_for_status()
                yield response

        except Exception as error:
            logger.warning("POSTFILE ERROR %s", error)
            return None


def dagget(path: str):
    REST_URL = f"https://{demostore['host']}:8243{path}"
    # logger.debug("DAG GET %s", REST_URL)
    try:
        response = requests.get(url=REST_URL, auth=AUTH_CREDENTIALS, verify=False)
        # logger.debug("DAG GET RESPONSE: %s", response)
        response.raise_for_status()
        return response

    except Exception as error:
        logger.warning("DAG GET ERROR %s", error)
        return None


def dagput(path: str, data=None):
    REST_URL = f"https://{demostore['host']}:8243{path}"
    # logger.debug("DAG PUT: %s", REST_URL)
    try:
        response = requests.put(url=REST_URL, auth=AUTH_CREDENTIALS, verify=False, data=data)
        # logger.debug("DAG PUT RESPONSE: %s", response)
        response.raise_for_status()
        return response

    except requests.exceptions.HTTPError as httperror:
        if httperror.response.status_code != 409:
            logger.warning("DAG PUT HTTP ERROR %s", httperror.response.text)

    except Exception as error:
        logger.warning("DAG PUT ERROR %s", error)
        return None


def dagpost(path: str, json_obj=None):
    REST_URL = f"https://{demostore['host']}:8243{path}"
    # logger.debug("DAGPOSTDATA: %s", type(json_obj))

    try:
        response = requests.post(
            url=REST_URL,
            auth=AUTH_CREDENTIALS,
            verify=False,
            json=json_obj,
            # headers={"Content-Type: application/json"},
        )
        # logger.debug("DAG POST RESPONSE: %s", response)
        response.raise_for_status()
        return response

    except Exception as error:
        logger.warning("DAG POST ERROR %s", error)
        return None


def kafkaget(path: str):
    REST_URL = f"https://{demostore['host']}:8082{path}"
    # logger.debug("KAFKA GET %s", REST_URL)
    try:
        response = requests.get(
            url=REST_URL,
            auth=AUTH_CREDENTIALS,
            verify=False,
            headers={
                "Content-Type": "application/vnd.kafka.json.v2+json",
                "Accept": "application/vnd.kafka.json.v2+json",
            },
        )
        # logger.debug("KAFKA GET RESPONSE: %s", response.text)
        response.raise_for_status()
        return response

    except Exception as error:
        # print(f"KAFKA GET ERROR {error}")
        logger.warning("KAFKA GET ERROR %s", error)
        return None


def kafkaput(path: str, data=None):
    REST_URL = f"https://{demostore['host']}:8082{path}"
    # logger.debug("KAFKA PUT: %s", REST_URL)
    try:
        response = requests.put(
            url=REST_URL,
            auth=AUTH_CREDENTIALS,
            verify=False,
            data=json.dumps({"records": [{"value": data}]}),
            headers={"Content-Type": "application/vnd.kafka.json.v2+json"},
        )
        # logger.debug("KAFKA PUT RESPONSE: %s", response)
        response.raise_for_status()
        return response

    except Exception as error:
        logger.warning("KAFKA PUT ERROR %s", error)
        return None


def kafkapost(path: str, data=None):
    REST_URL = f"https://{demostore['host']}:8082{path}"
    # logger.debug("KAFKA POST DATA: %s", data)

    try:
        response = requests.post(
            url=REST_URL,
            auth=AUTH_CREDENTIALS,
            verify=False,
            data=json.dumps(data),
            headers={ "Content-Type": "application/vnd.kafka.json.v2+json" },
        )
        # logger.debug("KAFKA POST RESPONSE: %s", response.content)
        response.raise_for_status()
        return response

    except Exception as error:
        # print(error.text)
        logger.warning("KAFKA POST ERROR %s", error)
        return None


def kafkadelete(path: str):
    REST_URL = f"https://{demostore['host']}:8082{path}"

    try:
        response = requests.delete(
            url=REST_URL,
            auth=AUTH_CREDENTIALS,
            verify=False,
            headers={"Content-Type": "application/vnd.kafka.v2+json"},
        )
        # logger.debug("KAFKA DELETE REQUEST: %s", response.request)
        # logger.debug("KAFKA DELETE RESPONSE FOR %s: %s", REST_URL, response.text)
        response.raise_for_status()
        return response

    except requests.exceptions.HTTPError as httperror:
        if httperror.response.status_code != 404:
            logger.warning("KAFKA DELETE HTTP ERROR %s", httperror.response.text)

    except Exception as error:
        # print(f"KAFKA DELETE ERROR {error}")
        logger.warning("KAFKA DELETE ERROR %s", error)
        return None


# Not used - basic auth used instead
# def get_session():
#     REST_URL = f"https://{demostore['target']}:8443/"
#     AUTH_CREDENTIALS = (
#         demostore["username"],
#         demostore["password"],
#     )
#     try:
#         return requests.get(url=REST_URL, auth=AUTH_CREDENTIALS, verify=False)

#     except Exception as error:
#         return error



import pyspark
from delta import *

appName = "Stage_1"
callTable = "/telecom/calltable"
subscriberTable = "/telecom/subscribertable"
targetTable = "/telecom/staging_1"
targetCheckPointPath = targetTable + "/_checkpoints/stage_1"


def main():
    builder = pyspark.sql.SparkSession.builder.appName(appName) \
        .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension") \
        .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog") 

    spark = configure_spark_with_delta_pip(builder).getOrCreate()

    subscribers = spark.read.format("delta").load(subscriberTable).alias('subscribers')
    calls = spark.readStream.format("delta").load(callTable).alias('calls')

    tab = calls.join(subscribers, calls["userid"] == subscribers["user"]) \
        .select("user","firstname","lastname","startime","endtime","latitude","longitude") \
        .writeStream \
        .format("delta")  \
        .option("mergeSchema", "true") \
        .option("checkpointLocation", targetCheckPointPath) \
        .start(targetTable)
    
    #staging_1 = DeltaTable.forPath(spark, targetTable)
    #staging_1.vacuum(0.1)

    tab.awaitTermination()

if __name__ == '__main__':
    main()



import io
import csv
import json
import time
import avro.schema
from avro.io import DatumWriter
from datetime import datetime
#from kafka import KafkaProducer
from confluent_kafka import Producer

def getTime(str):
    if ( len(str) == 10 ):
         str = str + " 00:00"
    return int(datetime.strptime(str,'%d/%m/%Y %H:%M').timestamp())

def buildJSON(row,id):
    return {
        "id": id,
        "startime": getTime(row[0]),
        "endtime": getTime(row[1]),
        "base": row[2],
        "userid": row[3]
    }

count = 0
schema = avro.schema.parse(open("./resources/call.avsc", 'r').read())
#producer = KafkaProducer( bootstrap_servers = ['maprdemo.mapr.io:9092'] )
producer = Producer( {'streams.producer.default.stream': '/telecom/mystream'} )

with open("./data/calls.csv", 'r') as csvfile:
    reader = csv.reader(csvfile, delimiter=',')
    header = next(reader)
    for row in reader:
        data = buildJSON(row,str(count))
        writer = DatumWriter(schema)
        bytesWriter = io.BytesIO()
        encoder = avro.io.BinaryEncoder(bytesWriter)
        writer.write( data, encoder)          
        producer.produce( "calls", key=data['id'], value = bytesWriter.getvalue()  )
        producer.flush()
        count = count + 1



# Config show dialog
import json
import nicegui
from ezlab.parameters import DF
from ezlab.pages import ezmeral

from ezlab.pages.sshkeys import ssh_keys_ui
from ezlab.pages import targets
from ezlab.pages import settings

appstore = nicegui.app.storage.general

def save_config(val: str, dialog):
    try:
        for key, value in json.loads(val.replace("\n", "")).items():
            appstore[key] = value
        for m in (ezmeral.menu, targets.menu, settings.menu):
            m.refresh()
        dialog.close()
        nicegui.ui.notify("Settings loaded", type="positive")
    except (TypeError, json.decoder.JSONDecodeError, ValueError) as error:
        nicegui.ui.notify("Not a valid json", type="negative")
        print(error)


def config_show():
    with nicegui.ui.dialog() as config_show, nicegui.ui.card().classes("w-full h-full"):
        config_json = json.dumps(appstore, indent=2)
        nicegui.ui.code(config_json, language="json").classes("w-full text-wrap")
        with nicegui.ui.row():
            nicegui.ui.button(
                "Save (useless on app mode)",
                icon="save",
                on_click=lambda: nicegui.ui.download(
                    config_json.encode(), "ezlab-settings.json"
                ),
            )
            nicegui.ui.button(
                "Close",
                icon="cancel",
                on_click=config_show.close,
            )
    return config_show


# Config load dialog
def config_load():
    with nicegui.ui.dialog() as config_load, nicegui.ui.card().classes("w-full h-full"):
        jsontext = (
            nicegui.ui.textarea()
            .props("stack-label=json autogrow filled")
            .classes("h-dvh w-full text-wrap")
        )
        with nicegui.ui.row():
            nicegui.ui.button(
                "Save",
                on_click=lambda _: save_config(
                    jsontext.value,
                    config_load,
                ),
                icon="save",
            )
            nicegui.ui.button(
                "Close",
                icon="cancel",
                on_click=config_load.close,
            )

    return config_load

@nicegui.ui.refreshable
def menu():
    with (
        nicegui.ui.expansion(
            "Settings",
            icon="settings",
            caption="for your environment",
        )
        .classes("w-full")
        .classes("text-bold") as settings
    ):
        nicegui.ui.checkbox("Dry Run", value=False).bind_value(
            appstore["config"], "dryrun"
        )
        nicegui.ui.label("Network Settings").classes("text-bold")
        with nicegui.ui.row():
            nicegui.ui.input("VM Network", placeholder="10.1.1.0/24").bind_value(
                appstore["config"], "cidr"
            )
            nicegui.ui.input("Gateway", placeholder="10.1.1.1").bind_value(
                appstore["config"], "gateway"
            )
            nicegui.ui.input("Name Server", placeholder="10.1.1.1").bind_value(
                appstore["config"], "nameserver"
            )
            nicegui.ui.input("Domain", placeholder="ez.lab").bind_value(
                appstore["config"], "domain"
            )
            nicegui.ui.input("HTTP Proxy", placeholder="").bind_value(
                appstore["config"], "proxy"
            )

        nicegui.ui.label("Repositories").classes("text-bold")
        with nicegui.ui.row().classes("w-full"):
            # YUM Repo
            nicegui.ui.input("YUM Repo", placeholder="").bind_value(
                appstore["config"], "yumrepo"
            )

            nicegui.ui.input("EPEL Repo", placeholder="").bind_value(
                appstore["config"], "epelrepo"
            )

        # MapR Repository Configuration
        with nicegui.ui.row().classes("w-full"):
            nicegui.ui.label("MapR Repo").classes("self-center")
            switch = (
                nicegui.ui.switch("Use local", value=False)
                .props("left-label")
                .bind_value(appstore[DF], "maprrepoislocal")
            )

            # Using default HPE repository
            with (
                nicegui.ui.row()
                .bind_visibility_from(switch, "value", lambda x: not x)
                .classes("w-full items-justify")
            ):
                nicegui.ui.input("HPE Passport e-mail").bind_value(
                    appstore[DF], "maprrepouser"
                )
                nicegui.ui.input(
                    "HPE Passport token", password=True, password_toggle_button=True
                ).bind_value(appstore[DF], "maprrepotoken")

            # Using local repository
            with nicegui.ui.row().bind_visibility_from(switch, "value"):
                nicegui.ui.input(
                    "MapR Repo",
                    placeholder="https://repo.ez.lab/mapr/",
                ).bind_value(appstore[DF], "maprlocalrepo")
                authlocal = nicegui.ui.switch("Authenticate", value=True).bind_value(
                    appstore[DF], "maprlocalrepoauth"
                )
                with nicegui.ui.row():
                    nicegui.ui.input("Username", password=True).bind_value(
                        appstore[DF], "maprlocalrepousername"
                    ).bind_visibility(authlocal, "value")
                    nicegui.ui.input("Password", password=True).bind_value(
                        appstore[DF], "maprlocalrepopassword"
                    ).bind_visibility(authlocal, "value")

        nicegui.ui.label("Cloudinit Settings").classes("text-bold")
        with nicegui.ui.row():
            nicegui.ui.input("username", placeholder="ezmeral").bind_value(
                appstore["config"], "username"
            )
            nicegui.ui.input("password", password=True, password_toggle_button=True).bind_value(
                appstore["config"], "password"
            )

        nicegui.ui.label("SSH Key").classes("text-bold")
        ssh_keys_ui()

    settings.bind_value(appstore["ui"], "settings")



import os

from ezlab.parameters import DF, GENERIC, UA

ini_file = os.path.expanduser("~/.ezconfig.ini")

# class ez_product(str, Enum):
#     df = "ezdf"
#     ua = "ezua"
#     generic = "generic"

# ez_nodes = [
#     {
#         "name": "ua-control",
#         "product": UA,
#         "cores": 4,
#         "memGB": 8,
#         "os_disk_size": 300,
#         "data_disk_size": 500,
#         "no_of_disks": 1,
#         "count": 2,
#     },
#     {
#         "name": "ua-worker",
#         "product": UA,
#         "cores": 32,
#         "memGB": 128,
#         "os_disk_size": 300,
#         "data_disk_size": 500,
#         "no_of_disks": 2,
#         "count": 3,
#     },
#     {
#         "name": "df-singlenode",
#         "product": DF,
#         "cores": 32,
#         "memGB": 96,
#         "os_disk_size": 200,
#         "data_disk_size": 500,
#         "no_of_disks": 1,
#         "count": 1,
#     },
#     {
#         "name": "df-5nodes",
#         "product": DF,
#         "cores": 16,
#         "memGB": 34,
#         "os_disk_size": 200,
#         "data_disk_size": 500,
#         "no_of_disks": 1,
#         "count": 5,
#     },
#     {
#         "name": "client",
#         "product": GENERIC,
#         "cores": 2,
#         "memGB": 4,
#         "os_disk_size": 50,
#         "no_of_disks": 0,
#         "count": 1,
#     },
#     {
#         "name": "generic",
#         "product": GENERIC,
#         "cores": 1,
#         "memGB": 2,
#         "os_disk_size": 20,
#         "no_of_disks": 0,
#         "count": 1,
#     },
# ]


ez_files = {
    DF: {
        "/etc/sysctl.d/mapr.conf": "vm.swappiness=1\n",
        "/etc/profile.d/mapr.sh": "umask 0022\n",
    },
    UA: {},
    GENERIC: {},
}

# ez_commands = {
#     DF: [
#         # "sudo chown root:root /etc/sudoers.d/mapr -- 2>/dev/null && sudo chmod 440 /etc/sudoers.d/mapr",
#         # disable subscription manager - TODO: needs testing
#         "[ -f /etc/yum/pluginconf.d/product-id.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/product-id.conf",
#         "[ -f /etc/yum/pluginconf.d/subscription-manager.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/subscription-manager.conf",
#         "sudo sysctl vm.swappiness=1 > /dev/null",
#         # "sudo systemctl disable --now numad",
#         "sudo sed -i 's/^[^#]*PasswordAuthentication[[:space:]]no/PasswordAuthentication yes/' /etc/ssh/sshd_config",
#         "sudo systemctl restart sshd",
#         "sudo dnf install -y -q wget > /dev/null",
#     ],
#     UA: [
#         # disable subscription manager - TODO: needs testing
#         "[ -f /etc/yum/pluginconf.d/product-id.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/product-id.conf",
#         "[ -f /etc/yum/pluginconf.d/subscription-manager.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/subscription-manager.conf",
#         "sudo dnf -q -y install nfs-utils policycoreutils-python-utils conntrack-tools jq tar >/dev/null",
#         "sudo dnf --setopt=tsflags=noscripts install -y -q iscsi-initiator-utils >/dev/null",
#         'echo "InitiatorName=$(/sbin/iscsi-iname)" | sudo tee -a /etc/iscsi/initiatorname.iscsi >/dev/null',
#         "sudo systemctl enable --now iscsid 2>&1 > /dev/null",
#         "sudo ethtool -K eth0 tx-checksum-ip-generic off >/dev/null",
#         "sudo sed -i 's/^[^#]*PermitRootLogin[[:space:]]no/PermitRootLogin yes/' /etc/ssh/sshd_config",
#         "sudo systemctl restart sshd",
#     ],
#     GENERIC: [
#         "[ -f /etc/yum/pluginconf.d/product-id.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/product-id.conf",
#         "[ -f /etc/yum/pluginconf.d/subscription-manager.conf ] && sudo sed -i 's/^enabled=0/enabled=1/' /etc/yum/pluginconf.d/subscription-manager.conf",
#         "sudo sed -i 's/^[^#]*PasswordAuthentication[[:space:]]no/PasswordAuthentication yes/' /etc/ssh/sshd_config",
#         "sudo systemctl restart sshd",
#     ],
# }

ez_installers = {
    DF: {
        "image": "maprtech/edf-seed-container:latest",
        "start_installer": "./datafabric_container_setup.sh",
        "url": "https://localhost:8443/app/dfui/#/installer",
        "commands": [],
    },
    UA: {
        "start_installer": "./start_ezua_installer_ui.sh",
        # "image": "marketplace.us1.greenlake-hpe.com/ezua/ezua/hpe-ezua-installer-ui:latest",
        "image": "us.gcr.io/mapr-252711/hpe-ezua-installer-ui:1.2.0-d16afb0",
        "url": "http://localhost:8080",
    },
    GENERIC: {},
}




# PRODUCER

# conf = {
#     "bootstrap.servers": f"{host}:9092",
#     # "queue.buffering.max.messages": 100000,
#     "acks": 1,
#     "queue.buffering.max.ms": 10,
#     "security.protocol": "SASL_PLAINTEXT",
#     "sasl.mechanism": "PLAIN",
#     "enable.ssl.certificate.verification": "false",
#     "sasl.username": "mapr",
#     "sasl.password": "mapr",
#     "client.id": socket.gethostname(),
# }
# producer = Producer(conf)

# def delivery_report(err, msg):
#     logging.debug(f"Processed: {err if err is not None else msg}")
#     app.storage.general["demo"]["counting"] += 1

# for msg in messages:
#     producer.poll(0)
#     producer.produce(topic, key="id", value=json.dumps(msg).encode(), callback=delivery_report)
#     # logging.debug(f"Sent: {msg}")

# producer.flush()


# CONSUMER
    # conf = {
    #     "bootstrap.servers": f"{host}:9092",
    #     "group.id": "ezshow-consumer",
    #     "auto.offset.reset": "earliest",
    #     "security.protocol": "SASL_PLAINTEXT",
    #     "sasl.mechanism": "PLAIN",
    #     "enable.ssl.certificate.verification": "false",
    #     "sasl.username": "mapr",
    #     "sasl.password": "mapr",
    # }
    # consumer = Consumer(conf)

    # running = True
    # logging.info(f"Getting messages from {topic}")
    # try:
    #     consumer.subscribe([topic])

    #     while running:
    #         msg = consumer.poll(timeout=1.0)
    #         if msg is None:
    #             continue

    #         if msg.error():
    #             if msg.error().code() == KafkaError._PARTITION_EOF:
    #                 # End of partition event, nothing to worry about, just log it
    #                 logging.debug(
    #                     "%% %s [%d] reached end at offset %d\n"
    #                     % (msg.topic(), msg.partition(), msg.offset())
    #                 )
    #             elif msg.error():
    #                 # not concerned about errors, just report it
    #                 yield msg.error()
    #         else:
    #             print(msg)
    #             yield msg
    # finally:
    #     # Close down consumer to commit final offsets.
    #     consumer.close()

    # yield "TASK_COMPLETED"



import asyncio
import nicegui
from proxmoxer import ProxmoxAPI
from libvirt import virConnect
import ezlab.utils as utils

from ezlab.parameters import KVM, PVE, SUPPORTED_HVES

from ezlab.pages.vms import new_vm_ui
from ezinfra import kvm, pve

appstore = nicegui.app.storage.general

@nicegui.ui.refreshable
def menu():
    userstore = nicegui.app.storage.user

    with (
        nicegui.ui.expansion("VMs", icon="computer", caption="create and configure")
        .classes("w-full")
        .classes("text-bold") as virtualmachines
    ):
        with nicegui.ui.row().classes("w-full items-center justify-between"):
            # Login dialog
            with nicegui.ui.dialog() as dialog, nicegui.ui.card():
                t = (
                    nicegui.ui.radio(options=SUPPORTED_HVES)
                    .classes("w-full")
                    .props("inline")
                    .bind_value(appstore["target"], "hve")
                )

                h = (
                    nicegui.ui.input("Host")
                    .classes("w-full")
                    .bind_value(appstore["target"], "host")
                )

                u = (
                    nicegui.ui.input("Username")
                    .classes("w-full")
                    .bind_value(appstore["target"], "username")
                )

                p = (
                    nicegui.ui.input(
                        "Password",
                        password=True,
                        password_toggle_button=True,
                    )
                    .classes("w-full")
                    .bind_value(appstore["target"], "password")
                    .bind_enabled_from( # disable password entry for kvm, only ssh key authorisation works (or user need to enter password at the cli)
                        appstore["target"], "hve", backward=lambda x: x != KVM
                    )
                )

                nicegui.ui.button(
                    "Login",
                    on_click=lambda: dialog.submit((t.value, h.value, u.value, p.value)),
                )

            # Connection view
            with nicegui.ui.row():
                nicegui.ui.button(icon="cloud_done", on_click=lambda: loginbuttonaction(dialog)).bind_visibility_from(
                    appstore["target"],
                    "connected",
                )
                nicegui.ui.button(
                    icon="cloud_off", on_click=lambda: loginbuttonaction(dialog)
                ).bind_visibility_from(
                    appstore["target"],
                    "connected",
                    lambda x: not x,
                )

            # new vm button
            nicegui.ui.label().bind_text_from(
                appstore["target"],
                "hve",
                backward=lambda x: f"{x} connected",
            ).bind_visibility_from(appstore["target"], "connected")
            nicegui.ui.button("New VM", on_click=new_vm_ui).bind_visibility_from(
                appstore["target"], "connected"
            ).bind_enabled_from(userstore, "busy", backward=lambda x: not x)

    virtualmachines.bind_value(appstore["ui"], "virtualmachines")


async def loginbuttonaction(loginform: nicegui.ui.dialog):
    if appstore["target"]["connected"]:
        # TODO: disconnect process/cleanup
        appstore["target"]["connected"] = False
   
    else:
        result = await loginform

        # modify password field if target is KVM, so emptiness check below can proceed without password
        if result[0] == KVM:
            result[3] == "nopasswordsupportforkvm"

        # emptiness check
        if result and all(result):
            connection = await connect(result)
            if connection: 
                utils.ezapp.connection = connection
                appstore["target"]["connected"] = True

        else:
            nicegui.ui.notify("Missing information", type='warning')


async def connect(params: set):
    target, host, username, password = params
    try:
        if target == PVE:
            result = await asyncio.to_thread(pve.connect, host, username, password)

        elif target == KVM:
            result = await asyncio.to_thread(kvm.connect, host, username)

        # elif target == VMWARE:
        #     return await asyncio.to_thread(vmw.connect, host, username, password)

        else:
            result = None

        if isinstance(result, ProxmoxAPI) or isinstance(result, virConnect):
            appstore["target"]["connected"] = True
            return result
        else:
            appstore["target"]["connected"] = False
            nicegui.ui.notify(f"FAILED: {result}", type="negative")

    except Exception as error:
        nicegui.ui.notify(f"CONNECT ERROR: {error}", type="negative")



import nicegui
from ezlab.parameters import EZNODES, KVM, PVE
from ezinfra.queue import process_queue
from ezinfra.vms import clone
from ezinfra import kvm, pve
from ezlab.utils import ezapp

appstore = nicegui.app.storage.general

def pve_newvm_dialog():
    userstore = nicegui.app.storage.user

    userstore["busy"] = True

    templates = [
        x for x in pve.vms(ezapp.connection) if x["template"] and x["type"] == "qemu"
    ]
    networks = pve.networks(ezapp.connection)
    storage = pve.storage(ezapp.connection)

    userstore["busy"] = False

    if "template" not in userstore.keys():
        userstore["template"] = templates[0]

    def set_template(name):
        userstore["template"] = [
            t for t in templates if t["name"] == name
        ].pop()

    # vm settings dialog
    with nicegui.ui.dialog() as dialog, nicegui.ui.card():
        template_selector = (
            nicegui.ui.select(
                options=[x["name"] for x in templates],
                label="Template",
                on_change=lambda e: set_template(e.value),
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "template")
        )
        storage_selector = (
            nicegui.ui.select(
                options=[
                    x["storage"]
                    for x in storage
                    if x["node"] == userstore["template"]["node"]
                ],
                label="Storage",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "storage")
        )
        network_selector = (
            nicegui.ui.select(
                options=[
                    x["sdn"]
                    for x in networks
                    if x["node"] == userstore["template"]["node"]
                ],
                label="Network",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "network")
        )

        bridge_selector = (
            nicegui.ui.select(
                options=[
                    x["iface"]
                    for x in pve.bridges(
                        ezapp.connection, userstore["template"]["name"]
                    )
                ],
                label="Bridge",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "bridge")
        )

        eznode_selector = (
            nicegui.ui.select(
                options=[x["name"] for x in EZNODES],
                label="Node Type",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "eznode")
        )

        host_selector = (
            nicegui.ui.input(label="VM Name", placeholder="ezvm")
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "hostname")
        )

        firstip_selector = (
            nicegui.ui.input(
                label="First IP",
                placeholder=appstore["config"]["gateway"],
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[PVE], "firstip")
        )

        nicegui.ui.button(
            "Create",
            on_click=lambda: dialog.submit(
                (
                    next(
                        (i for i in templates if i["name"] == template_selector.value),
                        None,
                    ),
                    next(
                        (i for i in storage if i["storage"] == storage_selector.value),
                        None,
                    ),
                    bridge_selector.value,
                    next(
                        (i for i in EZNODES if i["name"] == eznode_selector.value),
                        None,
                    ),
                    host_selector.value,
                    firstip_selector.value,
                )
            ),
        )

    return dialog


def kvm_newvm_dialog():
    userstore = nicegui.app.storage.user

    @nicegui.ui.refreshable
    def volume_selector():
        userstore["busy"] = True
        vols = kvm.volumes(
            appstore[KVM].get("pool", "default")
        )
        selector = (
            nicegui.ui.select(
                options=vols,
                label="Base Image",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "baseimg")
        )
        userstore["busy"] = False
        return selector

    # select dialog
    with nicegui.ui.dialog() as dialog, nicegui.ui.card():

        userstore["busy"] = True

        pools = kvm.pools()
        bridges = kvm.bridges()

        userstore["busy"] = False

        pool = (
            nicegui.ui.select(
                options=[p.name() for p in pools],
                label="Pool",
                on_change=volume_selector.refresh,
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "pool")
        )

        baseimg = volume_selector()

        bridge = (
            nicegui.ui.select(
                options=bridges,
                label="Bridge",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "bridge")
        )

        eznode_selector = (
            nicegui.ui.select(
                options=[x["name"] for x in EZNODES],
                label="Node Type",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "eznode")
        )

        host_selector = (
            nicegui.ui.input(label="VM Name", placeholder="ezvm")
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "hostname")
        )

        firstip_selector = (
            nicegui.ui.input(
                label="First IP",
                placeholder="10.1.1.21",
            )
            .classes("w-full")
            .props("inline")
            .bind_value(appstore[KVM], "firstip")
        )

        nicegui.ui.button(
            "Create",
            on_click=lambda: dialog.submit(
                (
                    pool.value,
                    baseimg.value,
                    bridge.value,
                    next(
                        (i for i in EZNODES if i["name"] == eznode_selector.value),
                        None,
                    ),
                    host_selector.value,
                    firstip_selector.value,
                )
            ),
        )
    return dialog


async def new_vm_ui():
    userstore = nicegui.app.storage.user 

    if appstore["target"]["hve"] == PVE:
        dialog = pve_newvm_dialog()
    elif appstore["target"]["hve"] == KVM:
        dialog = kvm_newvm_dialog()
    else:
        nicegui.ui.notify("not implemented")

    resources = await dialog

    if resources and all(resources):
        vmcount = resources[3]["count"]

        ezapp.queue.put(f"Cloning {vmcount} VM(s)...")
        for count in range(vmcount):
            try:
                result = clone(
                    target=appstore["target"]["hve"],
                    resources=resources,
                    settings=dict(appstore["config"]),
                    vm_number=(
                        0 if vmcount == 1 else count + 1
                    ),  # start from 1 not 0, use 0 for single vm
                    dryrun=appstore["config"].get("dryrun", True),
                )
                if result:
                    pass
                else:
                    nicegui.ui.notify(f"Clone failed for {count}!", type="warning")

            except Exception as error:
                nicegui.ui.notify(error, type="negative")

        userstore["busy"] = True
        res = await nicegui.run.io_bound(process_queue, vmcount)
        userstore["busy"] = False
        if res:
            nicegui.ui.notify(f"Finished with {vmcount} clone(s)", type="positive")
        else:
            nicegui.ui.notify("Failed with clones", type="negative")

    else:
        nicegui.ui.notify("VM creation cancelled", type="info")



import pyspark
from delta import *

appName = "IngestAccount"
fileType = "csv"
fileHeader = True
fileSchema = True
fileDelimiter = ","
fileLocation = "/telecom/data/subscribers.csv"
targetTable = "/telecom/subscribertable"
targetTableCheckPoint = targetTable + "/_checkpoints/kafka2Raw"

def main():

  builder = pyspark.sql.SparkSession.builder.appName(appName) \
    .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension") \
    .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")

  spark = configure_spark_with_delta_pip(builder).getOrCreate()

  df = spark.read.format(fileType) \
    .option("inferSchema", fileSchema) \
    .option("header", fileHeader) \
    .option("sep", fileDelimiter) \
    .load(fileLocation)

  df.write.format("delta").mode("overwrite").save(targetTable)
    
  df.show()
  

if __name__ == '__main__':
  main()



from minio import Minio
from minio.error import S3Error

from nicegui import app

appstore = app.storage.general["demo"]


# https://github.com/minio/minio-py
# file_uploader.py MinIO Python SDK example
def upload():
    # Create a client with the MinIO server playground, its access key
    # and secret key.
    client = Minio(
        appstore["host"],
        access_key="",
        secret_key="",
    )

    # Create bucket.
    client.make_bucket("my-bucket")

    # The file to upload, change this path if needed
    source_file = "/tmp/test-file.txt"

    # The destination bucket and filename on the MinIO server
    bucket_name = "python-test-bucket"
    destination_file = "my-test-file.txt"

    # Make the bucket if it doesn't exist.
    found = client.bucket_exists(bucket_name)
    if not found:
        client.make_bucket(bucket_name)
        print("Created bucket", bucket_name)
    else:
        print("Bucket", bucket_name, "already exists")

    # Upload the file, renaming it in the process
    client.fput_object(
        bucket_name,
        destination_file,
        source_file,
    )
    print(
        source_file,
        "successfully uploaded as object",
        destination_file,
        "to bucket",
        bucket_name,
    )




from datetime import datetime, timedelta
import logging
from random import random
from time import sleep
from urllib.parse import quote, urlparse
from faker import Faker
from nicegui import app
import json

from ezlab.parameters import TASK_FINISHED
from ezlab.utils import getdemo

from ezshow import restrunner

logger = logging.getLogger("apprunner")


def send_transactions(params: list, count: int):
    # there should be single parameter (topic) for this function
    topic = params[0]

    fake = Faker("en_US")
    transactions = []
    for i in range(count):
        transactions.append(
            {
                "id": i + 1,
                "sender_account": fake.iban(),
                "receiver_account": fake.iban(),
                "amount": round(fake.pyint(0, 10_000), 2),
                "currency": "GBP", #fake.currency_code(),
                "transaction_date": fake.past_datetime(start_date="-12M").timestamp(),
            }
        )

    # logger.debug("Creating monitoring table")
    demo = getdemo(app.storage.general["demo"]["name"])
    table_path = f"{demo['volume']}/{demo['table']}"
    stream_path = f"{demo['volume']}/{demo['stream']}"
    create_monitoring_table(table_path=table_path)

    logger.info("Sending %s messages to %s", len(transactions), topic)
    import timeit
    tic = timeit.default_timer()

    stream_publish(
        stream=stream_path,
        topic=topic,
        messages=transactions,
    )
    toc = timeit.default_timer()
    logger.info("It took %i seconds", toc - tic)


def process_transactions(params: list):
    topic = params[0]
    demo = getdemo(app.storage.general["demo"]["name"])
    stream_path = f"{demo['volume']}/{demo['stream']}"
    
    count = 0
    
    import timeit
    tic = timeit.default_timer()
    for message in stream_consume(
        stream=stream_path,
        topic=topic,
    ):

        if message == TASK_FINISHED:
            break

        # logger.debug(f"CONSUMING TRANSACTION: {message}")
        # yield message
        print(f"Simulating txn processing for {message['id']}")
        sleep(0.1)
        count += 1

    toc = timeit.default_timer()
    logger.info(f"Processed %s transactions in %i seconds", count, toc - tic)


def generate_calls(params: list, count: int):
    print(f"To be implemented for {params} to generate {count} messages")


def push_to_monitorDB(stats_json_str: str):
    stats_json = json.loads(stats_json_str)
    stats_json["_id"] = str(stats_json['ts'])
    # logger.debug(f"_ID: {stats_json['_id']}")

    ts = stats_json["ts"] / 1_000_000
    logger.debug(
        "Submit metrics at %s",
        datetime.utcfromtimestamp(ts).strftime("%Y-%m-%d %H:%M:%S"),
    )

    response = restrunner.dagpost(
        "/api/v2/table/%2Fapps%2Fezshowmonitor", json_obj=stats_json
    )

    # logger.debug("MONITOR PUSH RESPONSE: %s", response)

    if isinstance(response, Exception):
        logger.warning("MONITOR PUSH FAILED: %s", response)
        return


def create_monitoring_table(table_path):
    # create monitoring table if not exist
    path = quote(table_path, safe="")
    response = restrunner.dagput(f"/api/v2/table/{path}")
    if isinstance(response, Exception):
        if response.response.status_code != 409:
            logger.warning(f"MonitorDB creation FAILED: {response}")


def kafka_publish(host: str, username: str, password: str, topic: str, messages: list):
    from confluent_kafka import Producer

    # logger.info("Sending %d messages to %s topic", len(messages), topic)
    logger.info(f"Sending {len(messages)} messages to {topic} topic")

    conf = {
        "bootstrap.servers": f"{host}:9092",
        "security.protocol": "SASL_PLAINTEXT",
        "sasl.mechanism": "PLAIN",
        # "enable.ssl.certificate.verification": "false", # if needed with SASL_SSL
        "sasl.username": username,
        "sasl.password": password,
        "client.id": "ezshow",
        "queue.buffering.max.ms": 100,
        "statistics.interval.ms": 1000,
        "stats_cb": push_to_monitorDB,
        "socket.timeout.ms": 1000,
        "message.send.max.retries": 2,
        "default.topic.config": {"message.timeout.ms": 1000, "request.timeout.ms": 1000},
    }

    producer = Producer(**conf)

    # process messages that are already in the queue
    producer.flush()

    # update progress on each poll
    def delivery_report(err, msg):
        if err is None:
            app.storage.general["demo"]["counting"] += 1
            # logger.debug(f"KAFKA DELIVERY: {msg.value().decode()}")
        else:
            logger.debug(f"{err}")

    for msg in messages:
        # process the queue
        producer.poll(0)

        # send the message
        producer.produce(
            topic, key="id", value=json.dumps(msg).encode(), on_delivery=delivery_report
        )

        # logger.debug(f"KAFKA PRODUCED {msg}")

    # process the remaining messages in the queue
    producer.flush()


def kafka_consume(host: str, username: str, password: str, topic: str):
    from confluent_kafka import Consumer, KafkaError, KafkaException

    # logger.info(f"Getting messages from {topic}")
    logger.info(f"Getting messages from %s", topic)

    conf = {
        "bootstrap.servers": f"{host}:9092",
        "group.id": "ezshow",
        "auto.offset.reset": "earliest",
        "security.protocol": "SASL_PLAINTEXT",
        "sasl.mechanism": "PLAIN",
        # "enable.ssl.certificate.verification": "false", # if needed with SASL_SSL
        "sasl.username": username,
        "sasl.password": password,
        "statistics.interval.ms": 500,
        "stats_cb": push_to_monitorDB,  # collect statistics
    }

    c = Consumer(conf)

    # Subscribe to topics
    c.subscribe([topic])

    # return the messages, log the errors
    try:
        while True:
            msg = c.poll(timeout=1.0)
            if msg is None:  # we break when there are no more messages in the topic,
                # in real life, you would keep waiting even when there are no messages
                logger.debug(f"Nothing left in the topic {topic} for consumption")
                # continue
                break
            if msg.error():
                if msg.error().code() == KafkaError._PARTITION_EOF:
                    # End of partition event
                    logging.info(
                        "%% %s [%d] reached end at offset %d\n"
                        % (msg.topic(), msg.partition(), msg.offset())
                    )
                elif msg.error():
                    raise KafkaException(msg.error())
                logger.warning(msg.error())
                yield False  # provide feedback on failed message processing
            else:
                logger.debug("Consumer sending message %s", msg.value().decode())
                yield msg.value().decode()

    except Exception as error:
        logger.warning("KAFKA CONSUME ERROR %s", error)

    finally:
        c.close()
        yield TASK_FINISHED


def stream_publish(stream: str, topic: str, messages: list):
    topic_path = quote(f"{stream}:{topic}", safe="")
    for msg in messages:
        response = restrunner.kafkapost(path=f"/topics/{topic_path}", data={"records": [{"value": msg}]})
        if isinstance(response, Exception):
            logger.info(f"Message failed to push: {response}")
        else:
            # logger.debug(f"STREAM PUSH {msg}")
            app.storage.general["demo"]["counting"] += 1
    
    logger.info("%s messages are sent to %s:%s", len(messages), stream, topic)


def stream_consume(stream: str, topic: str):
    topic_path = f"{stream}:{topic}"

    try:
        # Create consumer instance
        response = restrunner.kafkapost(
            f"/consumers/{topic}_consumer_group",
            data={
                "name": f"{topic}_consumer_instance",
                "format": "json",
                "auto.offset.reset": "earliest",
            },
        )

        if not response:
            yield TASK_FINISHED
            return

        # logger.debug("CONSUMER INSTANCE CREATED: %s", response.text)

        # Expected output from preceding command
        # {
        # "instance_id":"my_consumer_instance",
        # "base_uri":"http://localhost:8082/consumers/my_json_consumer_group/instances/my_consumer_instance"
        # }
        consumer_instance = response.json()
        consumer_instance_path = urlparse(consumer_instance['base_uri']).path
        # logger.debug("CONSUMER INSTANCE PATH: %s", consumer_instance_path)

        # subscribe to consumer
        restrunner.kafkapost(f"{consumer_instance_path}/subscription", data={ "topics": [ topic_path ] })
        # No content in response

        # get records
        records = restrunner.kafkaget(f"{consumer_instance_path}/records")
        if records.ok:
            for message in records.json():
                # logger.debug("CONSUMER GOT MESSAGE: %s", message)
                yield message["value"]

        # Expected output from preceding command
        # [
        # {"key":null,"value":{"foo":"bar"},"partition":0,"offset":0,"topic":"jsontest"}
        # ]
    except Exception as error:
        logger.warning("STREAM CONSUMER ERROR %s", error)

    finally:
        # Unsubscribe from consumer instance
        restrunner.kafkadelete(
            f"/consumers/{topic}_consumer_group/instances/{topic}_consumer_instance"
        )

    yield TASK_FINISHED


# MONITOR FUNCTIONS SHOULD RETURN
# "name": dict_item["type"],
# "time": dict_item["time"],
# "values": [
#     {"Tx": dict_item["tx"]},
#     {"Rx": dict_item["rx"]},
# ],

def dt_from_iso(timestring: str):
    # Workaround since received timestring with " AM"/" PM" suffix is not parsed properly
    isPM = " PM" in timestring
    dt = datetime.strptime(
        timestring.replace(" AM", "").replace(" PM", ""), "%Y-%m-%dT%H:%M:%S.%f%z"
    )
    return dt + timedelta(hours=12) if isPM else dt


def topic_stats(topic: str):
    demo = getdemo(app.storage.general["demo"]["name"])
    stream_path = f"{demo['volume']}/{demo['stream']}"
    # topic_path = quote(f"{stream_path}:{topic}", safe="")

    response = restrunner.get(f"/rest/stream/topic/info?path={stream_path}&topic={topic}")
    if isinstance(response, Exception):
        logger.warning(f"Failed to get metrics for {topic}: {response}")
    else:
        metrics = response.json()
        # logger.debug("GOT METRICS %s", metrics)
        series = []
        for m in metrics["data"]:
            series.append({"publishedMsgs": m["maxoffset"]})
            series.append({"consumedMsgs": m["minoffsetacrossconsumers"]})
            series.append(
                {
                    "latestAgo(s)": (
                        datetime.now().astimezone() - dt_from_iso(m["maxtimestamp"])
                    ).total_seconds()
                }
            )
            series.append(
                {
                    "consumerLag(s)": (
                        dt_from_iso(m["maxtimestamp"])
                        - dt_from_iso(m["mintimestampacrossconsumers"])
                    ).total_seconds()
                }
            )
        # logger.info("Metrics %s", series)
        return {
            "name": topic,
            "time": datetime.fromtimestamp(metrics["timestamp"]/(10**3)).strftime("%H:%M:%S"),
            "values": series,
        }


def consumer_stats(topic: str):
    demo = getdemo(app.storage.general["demo"]["name"])
    stream_path = f"{demo['volume']}/{demo['stream']}"

    response = restrunner.get(
        f"/rest/stream/cursor/list?path={stream_path}&topic={topic}"
    )
    if isinstance(response, Exception):
        logger.warning(f"Failed to get metrics for {topic}: {response}")
    else:
        metrics = response.json()
        # logger.debug("GOT METRICS %s", metrics)
        series = []
        for m in metrics["data"]:
            series.append({ f"{m['consumergroup']}_{m['partitionid']}_lag(s)": float(m["consumerlagmillis"])/1000})
            series.append({ f"{m['consumergroup']}_{m['partitionid']}_offsetBehind": int(m["produceroffset"]) - int(m['committedoffset'])})
        # logger.info("Metrics %s", series)
        return {
            "name": "Consumer",
            "time": datetime.fromtimestamp(metrics["timestamp"]/(10**3)).strftime("%H:%M:%S"),
            "values": series,
        }


# Dummy function for informational steps
def noop():
    return True

# def get_jmx_metrics():
#     buffertimestamp = (
#         datetime.now() - timedelta(minutes=0)
#     ).timestamp() * 10**6  # convert to microseconds

#     # logger.debug(
#     #     "Getting metrics from %s",
#     #     datetime.utcfromtimestamp(buffertimestamp/1_000_000).strftime("%Y-%m-%d %H:%M:%S"),
#     # )

#     response = restrunner.dagget(
#         f"/api/v2/table/%2Fapps%2Fezshowmonitor?orderBy=ts&fromId={buffertimestamp}"
#     )

#     # logger.debug("MONITOR GET RESPONSE: %s", response)

#     if isinstance(response, Exception):
#         logger.warning("MONITOR GET FAILED: %s", response)
#         return

#     if response is not None:
#         docs = response.json().get("DocumentStream", list())
#         if len(docs) > 0:
#             # print(docs)
#             for dict_item in sorted(docs, key=lambda x: x['ts']):
#                 yield {
#                     "name": dict_item["type"],
#                     "time": datetime.fromtimestamp(dict_item["time"]).time(),
#                     "values": [
#                         {"name": "Messages", "value": dict_item["msg_cnt"]},
#                         # {"name": "Rx", "value": dict_item["rx"]},
#                     ],
#                 }
#                 # yield { "x_axis": dict_item['time'], "y_axis": [ dict_item['tx'], dict_item['rx'] ] }
#             ######

#             # write to kafta stream, get json, return specific metrics/fields

#             ######
#             # import pandas as pd
#             # df = pd.json_normalize(docs)

#             # # print(df['type'])
#             # # print(df['time'])
#             # # print(df['msg_cnt'])
#             # # print(df['topics.banking.partitions.0.txmsgs'])

#             # metrics = df.pivot(index="ts", columns=["type"], values="msg_cnt")
#             # print(metrics)
#             # yield metrics


# def null_metrics():
#     # fake = Faker()
#     df = pd.DataFrame(columns=("timestamp", "metricA", "metricB"))
#     df.loc[0] = [datetime().now().timestamp(), 10*10, 5*10]
#     yield df



import asyncio
import base64
from ipaddress import IPv4Address, ip_address
import json
import os
import pathlib
import shlex
import socket
import subprocess
import importlib.resources
from queue import Queue

class ModuleContext:
    connection = None
    queue = None

    def __init__(self) -> None:
        pass

ezapp = ModuleContext()
ezapp.queue = Queue(1000)


# Read the demos
demopath = f"{pathlib.Path().resolve()}/ezshow/demos"
demos: list = None

demos = json.loads(importlib.resources.read_text("ezshow", "demos.json"))
# with open(f"{demopath}/demos.json", "r") as f:
#     demos = json.load(f)


def getdemo(name: str):
    return [d for d in demos if d['name'] == name].pop()


# validation for non-zero entry
def nonzero(val):
    return len(val) != 0

# wrapper to make sync calls async-like
def fire_and_forget(f):
    def wrapped(*args, **kwargs):
        return asyncio.get_event_loop().run_in_executor(
            None, f, *args, *[v for v in kwargs.values()]
        )

    return wrapped

# find app on system path
def find_app(name: str):
    """Return executable path for `name` on PATH or in CWD."""
    from shutil import which

    return which(cmd=name, path=f"{os.environ.get('PATH')}:{os.getcwd()}")

# execute app/script locally
def execute(cmd: str):
    proc = subprocess.Popen(
        args=shlex.split(cmd),
        stdout=subprocess.PIPE,
        universal_newlines=True,
        # env=sub_env,
    )

    while proc.stdout:
        output = proc.stdout.readline().strip()
        if output == "" and proc.poll() is not None:
            break
        if output:
            yield f">>> {output}"
    rc = proc.poll()

# Base64 conversion for secrets etc
def toB64(string: str):
    return base64.b64encode(string.encode("ascii")).decode("utf-8")

# IP address validation
def validate_ip(ip_string: str) -> bool:
    try:
        return isinstance(ip_address(ip_string), IPv4Address)
    except:
        return False

# reliably get FQDN for given IP
def get_fqdn(ip_string: str) -> str:
    fqdn, _, _ = socket.gethostbyaddr(ip_string)
    return fqdn


def is_file(file):
    return os.path.isfile(os.path.abspath(file))



import pyspark
from delta import *

appName = "IngestBaseStation"
fileType = "csv"
fileHeader = True
fileSchema = True
fileDelimiter = ","
fileLocation = "/telecom/data/basestation.csv"
targetTable = "/telecom/basestations"
tableCheckpointPath = "/telecom/_checkpoints/csv2base"

def main():

  builder = pyspark.sql.SparkSession.builder.appName(appName) \
    .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension") \
    .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")

  spark = configure_spark_with_delta_pip(builder).getOrCreate()

  df = spark.read.format(fileType) \
    .option("inferSchema", fileSchema) \
    .option("header", fileHeader) \
    .option("sep", fileDelimiter) \
    .load(fileLocation)

#  df.write
#   .format("com.mapr.db.spark.streaming") \
#    .option("checkpointLocation", tableCheckpointPath) \
#    .option("tablePath", tablePath) \
#    .option("idFieldPath", "base") \
#    .option("bulkMode", True) \
#    .option("sampleSize", 100) \
#    .option("createTable", False) \
#    .mode("overwrite") \
#    .save(targetTable) 
    
  spark.saveToMapRDB(df, targetTable, id_field_path="id", create_table=False, bulk_insert=True)
  df.show()
  

if __name__ == '__main__':
  main()



import pyspark
from delta import *
from pyspark.sql.avro.functions import from_avro, to_avro

appName = "IngestRawTelemetry"
streamTopic = "/telecom/mystream:calls"
schemaPath = "./resources/call.avsc"
targetTable = "/telecom/callTable"
targetTableCheckPoint = targetTable + "/_checkpoints/kafka2Raw"



def main():
    builder = pyspark.sql.SparkSession.builder.appName(appName) \
        .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension") \
	.config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")

    spark = configure_spark_with_delta_pip(builder).getOrCreate()

    df = spark.readStream.format("kafka") \
	.option("kafka.bootstrap.servers", "localhost:9092" ) \
	.option("subscribe", streamTopic) \
	.option("startingOffsets", "earliest") \
	.option("failOnDataLoss", "false") \
        .option("maxFilesPerTrigger", 1) \
	.load()

    jsonFormatSchema = open(schemaPath,"r").read()

    rawData = df.select(from_avro("value",jsonFormatSchema).alias("call")) \
        .select("call.*") \
        .writeStream \
        .format("delta") \
        .outputMode("append") \
        .option("mergeSchema", "true") \
        .option("checkpointLocation",targetTableCheckPoint)  \
        .trigger(processingTime='2 seconds') \
        .start(targetTable)
  
    rawData.awaitTermination()


if __name__ == '__main__':
    main()



import pyspark
from delta import *
from pyspark.sql.avro.functions import from_avro, to_avro

tablePath = "/telecom/calls"
tableCheckpointPath = "/telecom/_checkpoints/kafka2DB"
streamTopic = "/telecom/mystream:calls"
schemaPath = "./resources/call.avsc"

builder = pyspark.sql.SparkSession.builder.appName("IngestRawTelemetry") \
    .config("spark.sql.extensions", "io.delta.sql.DeltaSparkSessionExtension") \
    .config("spark.sql.catalog.spark_catalog", "org.apache.spark.sql.delta.catalog.DeltaCatalog")

spark = configure_spark_with_delta_pip(builder).getOrCreate()

df = spark.readStream.format("kafka") \
    .option("kafka.bootstrap.servers", "localhost:9092" ) \
    .option("subscribe", streamTopic) \
    .option("startingOffsets", "earliest") \
    .option("failOnDataLoss", "false") \
    .load()

jsonFormatSchema = open( schemaPath, "r").read() 

rawData = df.select(from_avro("value",jsonFormatSchema).alias("call")) \
    .select("call.*") \
    .writeStream \
    .format("com.mapr.db.spark.streaming") \
    .option("checkpointLocation", tableCheckpointPath) \
    .option("tablePath", tablePath) \
    .option("idFieldPath", "id") \
    .option("bulkMode", True) \
    .option("sampleSize", 100) \
    .option("createTable", False) \
    .start()

rawData.awaitTermination()



import logging
from queue import Empty

from nicegui import app
from ezlab.utils import ezapp

from ezlab.parameters import TASK_FINISHED


def process_queue(number_of_tasks: int = 1):
    # Reset job count
    app.storage.general["ui"]["finished_tasks"] = 0

    while True:
        try:
            message = ezapp.queue.get_nowait()

            if message is not None:
                if TASK_FINISHED in message:
                    app.storage.general["ui"]["finished_tasks"] += 1
                    logging.info(
                        f"Completed jobs: {app.storage.general['ui']['finished_tasks']}"
                    )
                    if app.storage.general["ui"]["finished_tasks"] == number_of_tasks:
                        logging.info("All tasks completed")
                        app.storage.general["ui"]["finished_tasks"] = 0
                        ezapp.queue.task_done()
                        return True
                else:
                    if "error" in message or "failed" in message:
                        logging.error(message)
                    else:
                        logging.info(message)
                    # less eager for polling the queue
                    # sleep(0.3)
        except Empty:
            pass
        except Exception as error:
            ezapp.queue.put(TASK_FINISHED)
            ezapp.queue.task_done()
            print(f"QUEUE PROCESSING ERROR: {error}")
            return False



import io
import avro.schema
import avro.io
from confluent_kafka import Consumer, KafkaError

def main():
    running = True
    topic = '/telecom/mystream:mytopic'
    schemaPath = '/mapr/maprdemo.mapr.io/mydata/resources/call.avsc'
    consumer = Consumer({'group.id': 'mygroup', 'default.topic.config': {'auto.offset.reset': 'earliest'}} )

    consumer.subscribe([topic])
    schema = avro.schema.parse(open(schemaPath, 'r').read())

    while running:
        msg = consumer.poll(timeout=1.0)
        if msg is None: continue
  
        if not msg.error():  
            bytes_reader = io.BytesIO(msg.value())
            decoder = avro.io.BinaryDecoder(bytes_reader)
            reader = avro.io.DatumReader(schema)
            data = reader.read(decoder)
            print('Received message: %s' % data)
        elif msg.error().code() != KafkaError._PARTITION_EOF:
            print(msg.error())
            running = False
    
    consumer.close()


if __name__ == "__main__":
    main()



import io
import logging
import random
import socket
import string
from time import sleep
from paramiko import (
    AuthenticationException,
    AutoAddPolicy,
    BadHostKeyException,
    RSAKey,
    SSHClient,
    SSHException,
)

from ezlab.parameters import PVE

logging.getLogger("paramiko").setLevel(logging.WARNING)

def wait_for_ssh(host, username, privatekey):
    client = SSHClient()
    client.set_missing_host_key_policy(AutoAddPolicy)
    sleep_time = 5
    try:
        while True:
            sleep(sleep_time)
            print(f"DEBUG: trying {username}@{host}")
            client.connect(
                host,
                username=username,
                # key_filename=privatekey,
                pkey=RSAKey.from_private_key(io.StringIO(privatekey)),
                timeout=sleep_time,
            )
            break
    except (
        BadHostKeyException,
        AuthenticationException,
        ConnectionResetError,
        SSHException,
        socket.error,
    ):
        print(f"{host} still waiting ssh...")
        sleep(sleep_time)
    except Exception as error:
        print(f"{host} failed ssh connection with {error}")
        return False
    finally:
        print(f"{host} ssh connection check done")
        client.close()

    return True


def ssh_run_command(
    host: str,
    username: str,
    keyfile: str,
    command: str,
):
    """
    Run commands over ssh using default credentials
    Params:
        <host>              : hostname/IP to SSH
        <username>          : username to authenticate
        <keyfile>           : private key for <username>
        <command>           : command to run
    Returns:
        Generator with stdout. Calling function should process output with a for loop.
        Prints out stderr directly into terminal.

    """
    client = SSHClient()
    client.set_missing_host_key_policy(AutoAddPolicy)
    command_output = None
    # print(f"Try connection to {host}")
    try:
        client.connect(
            host,
            username=username,
            # key_filename=keyfile,
            # pkey=RSAKey.from_private_key(io.StringIO(keyfile)),
            pkey=RSAKey.from_private_key_file(keyfile),
            timeout=60,
        )

        # if "hostname -f" not in command:
        #     yield f"CMD: {command}"

        _stdin, _stdout, _stderr = client.exec_command(command, get_pty=True)
        command_errors = _stderr.read().decode().strip()
        command_output = _stdout.read().decode().strip()

        if command_errors and command_errors != "":
            yield f"[CMDERR]: {command_errors}"
        if command_output and command_output != "":
            # should return only the command output
            yield command_output

    except Exception as error:
        print("SSHCMD exception: ", error)

    finally:
        client.close()


def ssh_content_into_file(
    host: str, username: str, keyfile: str, content: str, filepath: str
):
    random_eof = "".join(random.choices(string.ascii_uppercase + string.digits, k=10))
    command = f"""cat << {random_eof} | sudo tee {filepath} > /dev/null
{content}
{random_eof}"""
    return ssh_run_command(
        host=host,
        username=username,
        keyfile=keyfile,
        command=command,
    )



from nicegui import app

# from ezinfra.remote import ssh_run_command

appstore = app.storage.general["demo"]

HOST = appstore["host"]
USERNAME = appstore["username"]
PASSWORD = appstore["password"]
# TODO: change this to reflect the correct location
KEYFILE = app.storage.general["config"]["privatekeyfile"]

def run_command(command):

    # for out in ssh_run_command(HOST, USERNAME, KEYFILE, command):
    #     print(out)

    return f"[ {HOST} ] RUN: {command}"



from nicegui import app, ui
from ezlab.parameters import DF

from ezlab.pages import df


@ui.refreshable
def menu():
    with (
        ui.expansion(
            "Ezmeral",
            icon="analytics",
            caption="install, configure and use",
        )
        .classes("w-full")
        .classes("text-bold") as ezmeral
    ):
        ui.label("Data Fabric").classes("text-bold")

        with ui.stepper().props("vertical header-nav").classes("w-full") as stepper:
            with ui.step("Configure", icon=""):
                df.prepare_menu()
                with ui.stepper_navigation():
                    ui.button(
                        "Run",
                        icon="play_arrow",
                        on_click=lambda: df.prepare_action(stepper),
                    ).bind_enabled_from(
                        app.storage.user, "busy", backward=lambda x: not x
                    )
                    ui.button(
                        "Next",
                        icon="fast_forward",
                        on_click=stepper.next,
                    ).props("color=secondary")

            with ui.step("Install"):
                ui.input("Cluster Name", placeholder="ezlab").bind_value(
                    app.storage.general[DF], "cluster_name"
                ).classes("w-full")

                maprclusterhosts = (
                    ui.input(
                        "Hosts",
                        placeholder="One or more comma-separated IPs, i.e., 10.1.1.x,10.1.1.y",
                    )
                    .classes("w-full")
                    .bind_value(app.storage.general[DF], "maprclusterhosts")
                )

                ui.input(
                    "Data Disk(s)",
                    placeholder="One or more comma-separated raw disk path, i.e., /dev/sdb,/dev/vdb",
                ).bind_value(app.storage.general[DF], "maprdisks").classes("w-full")

                with ui.stepper_navigation():
                    ui.button(
                        "Run",
                        icon="play_arrow",
                        on_click=lambda: df.install_action(
                            maprclusterhosts.value, stepper
                        ),
                    ).bind_enabled_from(
                        app.storage.user, "busy", backward=lambda x: not x
                    )
                    ui.button(
                        "Next",
                        icon="fast_forward",
                        on_click=stepper.next,
                    ).props("color=secondary")
                    ui.button("Back", icon="fast_rewind", on_click=stepper.previous).props("flat")

            # with ui.step("Cross-Cluster"):
            #     ui.label("Configure two clusters to join global namespace")
            #     ui.input(
            #         "Local Cluster CLDB node", placeholder="core.ez.lab"
            #     ).bind_value(app.storage.general[DF], "crosslocalcldb").classes(
            #         "w-full"
            #     )
            #     ui.input(
            #         "Remote Cluster CLDB node", placeholder="edge.ez.lab"
            #     ).bind_value(app.storage.general[DF], "crossremotecldb").classes(
            #         "w-full"
            #     )
            #     ui.input("Cluster Admin User", placeholder="mapr").bind_value(
            #         app.storage.general[DF], "crossadminuser"
            #     ).classes("w-full")
            #     ui.input(
            #         "Cluster Admin Password",
            #         placeholder="mapr",
            #         password=True,
            #         password_toggle_button=True,
            #     ).bind_value(app.storage.general[DF], "crossadminpassword").classes(
            #         "w-full"
            #     )

            #     with ui.stepper_navigation():
            #         ui.button(
            #             "Run",
            #             icon="play_arrow",
            #             on_click=lambda: df.xcluster_action(stepper),
            #         ).bind_enabled_from(
            #             app.storage.user, "busy", backward=lambda x: not x
            #         )
            #         ui.button(
            #             "Next",
            #             icon="fast_forward",
            #             on_click=stepper.next,
            #         ).props("color=secondary")
            #         ui.button("Back", icon="fast_rewind", on_click=stepper.previous).props("flat")

            with ui.step("Client"):
                ui.label("Setup a client to a Data Fabric cluster")
                ui.input("Server", placeholder="Hostname or ip address").bind_value(
                    app.storage.general[DF], "connect_to"
                )
                ui.switch("Remote", value=False).bind_value(
                    app.storage.general[DF], "isclientremote"
                )
                ui.input("Client", placeholder="Hostname or ip address").bind_value(
                    app.storage.general[DF], "maprclient"
                ).bind_visibility_from(app.storage.general[DF], "isclientremote")
                with ui.stepper_navigation():
                    ui.button(
                        "Run",
                        icon="play_arrow",
                        on_click=lambda: df.client_action(stepper),
                    ).bind_enabled_from(
                        app.storage.user, "busy", backward=lambda x: not x
                    )
                    ui.button(
                        "Back", icon="fast_rewind", on_click=stepper.previous
                    ).props("flat")

    ezmeral.bind_value(app.storage.general["ui"], "ezmeral")
